/**
 * @author luc
 */
package net.urlrewriter;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.urlrewriter.configuration.RewriterConfiguration;
import net.urlrewriter.utilities.HttpContextFacade;

/**
 * @author luc
 *
 */
public class RewriteFilter implements Filter {

	private static final String CONFIG_FILE = "configuration";
	private RewriterEngine mEngine;
	private ServletContext mServletContext;
	private Log mLogger = LogFactory.getLog(RewriteFilter.class);
	/**
	 * 
	 */
	public RewriteFilter() {
		super();
	}

	/* (non-Javadoc)
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig pConfig) throws ServletException {
		try {
			String oConfigFile = pConfig.getInitParameter(CONFIG_FILE);
			if (oConfigFile == null) throw new ServletException("Must specify configuration file for filter " + pConfig.getFilterName() + " using key \"" + CONFIG_FILE + "\".");
			String oFilePath = pConfig.getServletContext().getRealPath(oConfigFile);
			RewriterConfiguration oConfig = RewriterConfiguration.loadFromFile(oFilePath);
			mEngine = new RewriterEngine(oConfig);
			mServletContext = pConfig.getServletContext();
		} catch (Exception e) {
			throw new ServletException(e.getMessage(), e);
		}
	}

	/* (non-Javadoc)
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(ServletRequest pRequest, ServletResponse pResponse,
			FilterChain pChain) throws IOException, ServletException {
		HttpServletResponse oResponse = new RewriteResponseWrapper((HttpServletResponse) pResponse, (HttpServletRequest) pRequest, mEngine, mServletContext);
		
		HttpContextFacade oFacade = new HttpContextFacade(mServletContext, (HttpServletRequest) pRequest, oResponse);
		boolean oResponseCommitted = false;
		try {
			oResponseCommitted = mEngine.rewrite(oFacade);
		} catch (RewriteException e) {
			mLogger.fatal("Error rewriting url.", e);
			throw new ServletException(e.getMessage(), e);
		}
		if (!oResponseCommitted) {
			if (mLogger.isDebugEnabled()) mLogger.debug("Run chain...");
			pChain.doFilter(pRequest, oResponse);
		}
		if (mLogger.isDebugEnabled()) mLogger.debug("Done filtering request.");
	}

	/* (non-Javadoc)
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {
		
	}

}
