/**
 * @author luc
 */
package net.urlrewriter.actions;

import java.util.ArrayList;
import java.util.List;

import net.urlrewriter.RewriteContext;
import net.urlrewriter.RewriteException;
import net.urlrewriter.RewriteProcessing;
import net.urlrewriter.conditions.IRewriteCondition;

/**
 * @author luc
 *
 */
public class ConditionalAction implements IRewriteAction, IRewriteCondition {

	private List mActions = new ArrayList();
	private List<IRewriteCondition> mConditions = new ArrayList<IRewriteCondition>();
	
	/**
	 * 
	 */
	public ConditionalAction() {
		super();
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.actions.IRewriteAction#execute(net.urlrewriter.RewriteContext)
	 */
	public RewriteProcessing execute(RewriteContext pContext) throws RewriteException {
		RewriteProcessing oResult = RewriteProcessing.ContinueProcessing;
		for (int i = 0; i < mActions.size() && oResult == RewriteProcessing.ContinueProcessing; i++) {
			Object oItem = mActions.get(i);
			if (oItem instanceof IRewriteCondition) {
				IRewriteCondition oCondition = (IRewriteCondition) oItem;
				if (!oCondition.isMatch(pContext)) continue;
			}	
			IRewriteAction oAction = (IRewriteAction) mActions.get(i);
			oResult = oAction.execute(pContext);		
		}
		return oResult;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.conditions.IRewriteCondition#isMatch(net.urlrewriter.RewriteContext)
	 */
	public boolean isMatch(RewriteContext pContext) {
		for (IRewriteCondition oCondition : mConditions) {
			if (!oCondition.isMatch(pContext)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @return Returns the conditions.
	 */
	public List<IRewriteCondition> getConditions() {
		return mConditions;
	}

	/**
	 * @return Returns the actions.
	 */
	public List getActions() {
		return mActions;
	}

}
