/**
 * @author luc
 */
package net.urlrewriter.actions;

import java.util.ArrayList;
import java.util.List;

import net.urlrewriter.RewriteContext;
import net.urlrewriter.RewriteException;
import net.urlrewriter.RewriteProcessing;
import net.urlrewriter.conditions.IRewriteCondition;

/**
 * @author luc
 *
 */
public class RedirectAction extends SetLocationAction implements IRewriteCondition {

	private boolean mPermanent;
	private List<IRewriteCondition> mConditions = new ArrayList<IRewriteCondition>();
	
	/**
	 * @param pLocation
	 */
	public RedirectAction(String pLocation, boolean pPermanent) {
		super(pLocation);
		mPermanent = pPermanent;
	}

	@Override
	public RewriteProcessing execute(RewriteContext pContext) throws RewriteException {
		super.execute(pContext);
		if (mPermanent) {
			pContext.setHttpStatusCode(301); // Moved
		} else {
			pContext.setHttpStatusCode(302); // Found
		}
		return RewriteProcessing.StopProcessing;
	}
	
	/* (non-Javadoc)
	 * @see net.urlrewriter.conditions.IRewriteCondition#isMatch(net.urlrewriter.RewriteContext)
	 */
	public boolean isMatch(RewriteContext pContext) {
		for (IRewriteCondition oCondition : mConditions) {
			if (!oCondition.isMatch(pContext)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @return Returns the conditions.
	 */
	public List<IRewriteCondition> getConditions() {
		return mConditions;
	}

}
