/**
 * @author luc
 */
package net.urlrewriter.conditions;

import java.util.regex.Matcher;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.urlrewriter.RewriteContext;

/**
 * @author luc
 *
 */
public class PropertyMatchCondition extends MatchCondition {

	private String mPropertyName;
	private Log mLogger = LogFactory.getLog(PropertyMatchCondition.class);
	
	/**
	 * @param pPattern
	 */
	public PropertyMatchCondition(String pPropertyName, String pPattern) {
		super(pPattern);
		mPropertyName = pPropertyName;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.conditions.MatchCondition#isMatch(net.urlrewriter.RewriteContext)
	 */
	@Override
	public boolean isMatch(RewriteContext pContext) {
		String oProperty = pContext.getProperties().get(mPropertyName);
		if (mLogger.isDebugEnabled()) mLogger.debug("Checking property \"" + mPropertyName + "\".");
		if (oProperty != null) {
			Matcher oMatcher = getPattern().matcher(oProperty);
			if (oMatcher.matches()) {
				pContext.setLastMatch(oMatcher.toMatchResult());
				return true;
			} else {
				if (mLogger.isDebugEnabled()) mLogger.debug("Value " + oProperty + " didn't match pattern " + getPattern().pattern());
			}
		}
		return false;
	}

	/**
	 * @return Returns the propertyName.
	 */
	public String getPropertyName() {
		return mPropertyName;
	}

	/**
	 * @param pPropertyName The propertyName to set.
	 */
	public void setPropertyName(String pPropertyName) {
		mPropertyName = pPropertyName;
	}

}
