/**
 * @author luc
 */
package net.urlrewriter.conditions;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.urlrewriter.RewriteContext;

/**
 * @author luc
 *
 */
public class UrlMatchCondition implements IRewriteCondition {

	private String mPattern;
	private Pattern mRegex;
	private Log mLogger = LogFactory.getLog(UrlMatchCondition.class);
	
	/**
	 * 
	 */
	public UrlMatchCondition(String pPattern) {
		super();
		mPattern = pPattern;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.conditions.IRewriteCondition#isMatch(net.urlrewriter.RewriteContext)
	 */
	public boolean isMatch(RewriteContext pContext) {
		if (mRegex == null) {
			mRegex = Pattern.compile(pContext.resolveLocation(mPattern), Pattern.CASE_INSENSITIVE);
		}
		Matcher oMatcher = mRegex.matcher(pContext.getLocation());
		if (oMatcher.matches()) {
			if (mLogger.isDebugEnabled()) mLogger.debug("Url \"" + pContext.getLocation() + "\" matches pattern \"" + mPattern + "\".");
			pContext.setLastMatch(oMatcher.toMatchResult());
			return true;
		} else {
			if (mLogger.isDebugEnabled()) mLogger.debug("Url \"" + pContext.getLocation() + "\" does not match pattern \"" + mPattern + "\".");
			return false;
		}
	}

	/**
	 * @return Returns the pattern.
	 */
	public String getPattern() {
		return mPattern;
	}

	/**
	 * @param pPattern The pattern to set.
	 */
	public void setPattern(String pPattern) {
		mPattern = pPattern;
	}

}
