/**
 * @author luc
 */
package net.urlrewriter.configuration;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import net.urlrewriter.parsers.IRewriteActionParser;

/**
 * @author luc
 *
 */
public class ActionParserFactory {

	private Map<String, List<IRewriteActionParser>> mParsers;
	/**
	 * 
	 */
	public ActionParserFactory() {
		super();
		mParsers = new Hashtable<String, List<IRewriteActionParser>>();
	}

	public void addParser(String pClassName) throws ClassNotFoundException, InstantiationException, IllegalAccessException {
		Class oClass =  Class.forName(pClassName);
		IRewriteActionParser oParser = (IRewriteActionParser) oClass.newInstance();
		addParser(oParser);
	}
	
	public void addParser(IRewriteActionParser pParser) {
		List<IRewriteActionParser> oParsers;
		if (mParsers.containsKey(pParser.getName())) {
			oParsers = mParsers.get(pParser.getName()); 
		} else {
			oParsers = new ArrayList<IRewriteActionParser>();
			mParsers.put(pParser.getName(), oParsers);
		}
		oParsers.add(pParser);
	}
	
	public List<IRewriteActionParser> getParsers(String pVerb) {
		if (pVerb != null && mParsers.containsKey(pVerb)) {
			return mParsers.get(pVerb);
		} else {
			return null;
		}
	}
}
