/**
 * @author luc
 */
package net.urlrewriter.parsers;

import java.util.List;

import net.urlrewriter.actions.ConditionalAction;
import net.urlrewriter.actions.IRewriteAction;
import net.urlrewriter.configuration.ConfigurationException;
import net.urlrewriter.configuration.RewriterConfiguration;
import net.urlrewriter.utilities.Constants;
import net.urlrewriter.utilities.Messages;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Node;

/**
 * @author luc
 *
 */
public class ConditionActionParser extends RewriteActionParserBase {

	private Log mLogger = LogFactory.getLog(ConditionActionParser.class);
	
	/**
	 * 
	 */
	public ConditionActionParser() {
		super();
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.parsers.RewriteActionParserBase#parse(org.w3c.dom.Node, java.lang.Object)
	 */
	@Override
	public IRewriteAction parse(Node pNode, RewriterConfiguration pConfig)
			throws ConfigurationException {
		ConditionalAction oRule = new ConditionalAction();
		// Process the conditions on the element.
		boolean oNegative = (Constants.ELEMENT_IFNOT.equals(pNode.getNodeName()));
		if (mLogger.isDebugEnabled()) mLogger.debug("Parsing conditions...");
		parseConditions(pNode, oRule.getConditions(), oNegative, pConfig);

		// Next, process the actions on the element.
		readActions(pNode, oRule.getActions(), pConfig);

		return oRule;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.parsers.RewriteActionParserBase#getName()
	 */
	@Override
	public String getName() {
		return Constants.ELEMENT_IF;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.parsers.RewriteActionParserBase#getAllowNestedActions()
	 */
	@Override
	public boolean getAllowNestedActions() {
		return true;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.parsers.RewriteActionParserBase#getAllowAttributes()
	 */
	@Override
	public boolean getAllowAttributes() {
		return true;
	}

	private void readActions(Node pNode, List<IRewriteAction> pActions, RewriterConfiguration pConfig) throws ConfigurationException {
		Node oChildNode = pNode.getFirstChild();
		while (oChildNode != null) {
			if (oChildNode.getNodeType() == Node.ELEMENT_NODE) {
				if (mLogger.isDebugEnabled()) mLogger.debug("Reading action for node " + oChildNode.getNodeName());
				List<IRewriteActionParser> oParsers = pConfig.getActionParserFactory().getParsers(oChildNode.getNodeName());
				if (oParsers != null) {
					boolean oParsed = false;
					for (IRewriteActionParser oParser : oParsers) {
						if (mLogger.isDebugEnabled()) mLogger.debug("Parsing node " + oChildNode.toString() + " with parser " + oParser.getClass().getName());
						IRewriteAction oAction = oParser.parse(oChildNode, pConfig);
						if (oAction != null) {
							oParsed = true;
							pActions.add(oAction);
						}
					}
					if (!oParsed) {
						throw new ConfigurationException(String.format(Messages.ELEMENT_NOTALLOWED, oChildNode.getNodeName()));
					}
				}
			}
			oChildNode = oChildNode.getNextSibling();
		}
	}
}
