/**
 * @author luc
 */
package net.urlrewriter.parsers;

import java.util.List;

import net.urlrewriter.actions.IRewriteAction;
import net.urlrewriter.conditions.IRewriteCondition;
import net.urlrewriter.configuration.ConfigurationException;
import net.urlrewriter.configuration.RewriterConfiguration;
import net.urlrewriter.utilities.Constants;

import org.w3c.dom.Node;

/**
 * @author luc
 *
 */
public abstract class RewriteActionParserBase implements IRewriteActionParser {

	/**
	 * 
	 */
	public RewriteActionParserBase() {
		super();
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.parsers.IRewriteActionParser#parse(org.w3c.dom.Node, java.lang.Object)
	 */
	public abstract IRewriteAction parse(Node pNode, RewriterConfiguration pConfig) throws ConfigurationException;

	/* (non-Javadoc)
	 * @see net.urlrewriter.parsers.IRewriteActionParser#getName()
	 */
	public abstract String getName();

	/* (non-Javadoc)
	 * @see net.urlrewriter.parsers.IRewriteActionParser#getAllowNestedActions()
	 */
	public abstract boolean getAllowNestedActions();
	/* (non-Javadoc)
	 * @see net.urlrewriter.parsers.IRewriteActionParser#getAllowAttributes()
	 */
	public abstract boolean getAllowAttributes();
	
	protected void parseConditions(Node pNode, List<IRewriteCondition> pConditions, boolean pNegative, RewriterConfiguration pConfig) throws ConfigurationException {
		if (pConfig == null) return;
		
		// Parse attribute-based conditions.
		for (IRewriteConditionParser oParser : pConfig.getConditionParserPipeline()) {
			IRewriteCondition oCondition = oParser.parse(pNode);
			if (oCondition != null) {
				pConditions.add(oCondition);
			}
		}
		
		// Now, process the nested <and> conditions.
		Node oChildNode = pNode.getFirstChild();
		while (oChildNode != null) {
			if (oChildNode.getNodeType() == Node.ELEMENT_NODE) {
				if (Constants.ELEMENT_AND.equals(oChildNode.getNodeName())) {
					parseConditions(oChildNode, pConditions, pNegative, pConfig);
					// why are we doing this????
					Node oChildNode2 = oChildNode.getNextSibling();
					pNode.removeChild(oChildNode);
					oChildNode = oChildNode2;
					continue;
				}
			}
			oChildNode = oChildNode.getNextSibling();
		}
	}

}
