/**
 * @author luc
 */
package net.urlrewriter.utilities;

import java.io.IOException;
import java.net.URI;
import java.util.Enumeration;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.urlrewriter.errors.IRewriteErrorHandler;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author luc
 *
 */
public class HttpContextFacade implements IContextFacade {

	private HttpServletRequest mRequest;
	private HttpServletResponse mResponse;
	private ServletContext mServletContext;
	private Log mLogger = LogFactory.getLog(HttpContextFacade.class);
	private NameValueCollection mProperties;
	private NameValueCollection mVariables;
	private URI mURI = null;
	private String mRawUrl = null;
	
	/**
	 * 
	 */
	public HttpContextFacade(ServletContext pSContext, HttpServletRequest pRequest, HttpServletResponse pResponse) {
		super();
		mServletContext = pSContext;
		mRequest = pRequest;
		mResponse = pResponse;
		mProperties = new NameValueCollection();
		for (Enumeration<String> oEnum = pRequest.getHeaderNames(); oEnum.hasMoreElements();) {
			String oName = oEnum.nextElement();
			//String oValue = mProperties.getProperty(oName);
			//if (oValue != null) {
			//	oValue += "," + pRequest.getHeader(oName);
			//}
			mProperties.add(oName, pRequest.getHeader(oName));
		}
		mRawUrl = completeRawURL();
		try {
			mURI = new URI(completeURL());
		} catch (Exception e) {
			mLogger.fatal("Couldn't parse url \"" + mRequest.getRequestURI() + "\".", e);
		}
		
		mVariables = new NameValueCollection();
	}
	
	private String completeRawURL() {
		StringBuffer oURL = new StringBuffer(mRequest.getRequestURI());
		if (mRequest.getQueryString() != null && mRequest.getQueryString().length() > 0) {
			oURL.append("?").append(mRequest.getQueryString());
		}
		return oURL.toString();
	}
	
	private String completeURL() {
		StringBuffer oURL = mRequest.getRequestURL();
		if (mRequest.getQueryString() != null && mRequest.getQueryString().length() > 0) {
			oURL.append("?").append(mRequest.getQueryString());
		}
		return oURL.toString();
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#getApplicationPath()
	 */
	public String getApplicationPath() {
		return mRequest.getContextPath();
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#getRawUrl()
	 */
	public String getRawUrl() {
		return mRawUrl;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#getRequestUrl()
	 */
	public URI getRequestUrl() {
		return mURI;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#mapPath(java.lang.String)
	 */
	public String mapPath(String pUrl) {
		return mServletContext.getRealPath(pUrl);
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#setStatusCode(int)
	 */
	public void setStatusCode(int pCode) {
		mResponse.setStatus(pCode);
	}

	/**
	 * pUrl is an absolute url with 
	 */
	public void rewritePath(String pUrl) {
		String oUrl = pUrl;
		if (!oUrl.startsWith("/") || oUrl.startsWith(mRequest.getContextPath())) {
			int oIndex = oUrl.indexOf(mRequest.getContextPath());
			if (oIndex >= 0) {
				oUrl = oUrl.substring(oIndex + mRequest.getContextPath().length());
			}
		}
		//RequestDispatcher oDispatcher = mRequest.getRequestDispatcher(pUrl);
		RequestDispatcher oDispatcher = mRequest.getRequestDispatcher(oUrl);
		try {
			if (mLogger.isDebugEnabled()) mLogger.debug("Found dispatcher for url:" + oUrl + " (" + pUrl + ")");
			oDispatcher.forward(mRequest, mResponse);
		} catch (ServletException pSE) {
			mLogger.fatal("Couldn't forward to \"" + pUrl + "\".", pSE);
		} catch (IOException pIOE) {
			mLogger.fatal("Couldn't forward to \"" + pUrl + "\".", pIOE);
		} catch (Exception e) {
			mLogger.fatal("Coulnd't forward to \"" + pUrl + "\".", e);
		}
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#setRedirectLocation(java.lang.String)
	 */
	public void setRedirectLocation(String pUrl) {
		try {
			mResponse.sendRedirect(pUrl);
		} catch (IOException e) {
			mLogger.fatal("Couldn't redirect to \"" + pUrl + "\".", e);
		}
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#appendHeader(java.lang.String, java.lang.String)
	 */
	public void appendHeader(String pName, String pValue) {
		mResponse.addHeader(pName, pValue);
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#appendCookie(javax.servlet.http.Cookie)
	 */
	public void appendCookie(Cookie pCookie) {
		mResponse.addCookie(pCookie);
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#handleError(net.urlrewriter.errors.IRewriteErrorHandler)
	 */
	public void handleError(IRewriteErrorHandler pHandler) {
		pHandler.handleError(this);
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#SetItem(java.lang.Object, java.lang.Object)
	 */
	public void setItem(String pKey, Object pItem) {
		mRequest.setAttribute(pKey, pItem);
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#getItem(java.lang.Object)
	 */
	public Object getItem(String pKey) {
		return mRequest.getAttribute(pKey);
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#getHttpMethod()
	 */
	public String getHttpMethod() {
		return mRequest.getMethod();
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#getServerVariables()
	 */
	public NameValueCollection getServerVariables() {
		return mVariables;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#getHeaders()
	 */
	public NameValueCollection getHeaders() {
		return mProperties;
	}

	/* (non-Javadoc)
	 * @see net.urlrewriter.utilities.IContextFacade#getCookies()
	 */
	public Cookie[] getCookies() {
		return mRequest.getCookies();
	}

}
